package example.xlink.ddq;

import cn.xlink.ddq.common.type.ClientAuthType;
import cn.xlink.ddq.common.type.ClientModeType;
import cn.xlink.http2.client.callback.DdqClientMessageHandler;
import cn.xlink.http2.client.ddq.DdqH2Client;
import cn.xlink.http2.client.ddq.impl.DdqH2ClientBuilder;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;

/**
 * 使用订阅组方式进行DDQ连接，和接收数据。
 * 该客户端仅支持3.5.1以上的服务端。
 *
 * @author linsida linsida
 * @create 2020-08-07 10:08
 **/
@Slf4j
public class DdqH2ClientPullV2Demo {


    public static void main(String[] args) {
        try {
            //目标服务器IP或域名
            String host = "xxx.xxx.xxx.xxx";
            //目标服务服务器
            int port = 18996;

            //应用ID
            String         appId      = "320008bfe7a3dc00";
            String         appSecret  = "xxxx";
            ClientAuthType authType   = ClientAuthType.AccessKey;
            String         authHost   = "https://api-iot.xxx.xx";
            String         clientId   = "lock-service";
            String         clientSize = "1";
            String         threadNum  = "10";
            int            pullPeriod = 200;
            int            pullLimit  = 10000;
            String         groupId    = "xxxx-xx-xx-xx-xxx";


            log.info("======================= 连接信息展示 ================");
            log.info("目标机器：" + host + ":" + port);
            log.info("AppId：" + appId);
            log.info("AppSecret：" + appSecret);
            log.info("认证方式：" + authType);
            log.info("认证域名：" + authHost);
            log.info("客户端标识：" + clientId);
            log.info("客户端连接个数: " + clientSize);
            log.info("NettyIO线程个数: " + threadNum);
            log.info("拉取的间隔: " + pullPeriod + " ms");
            log.info("每次拉取的数量: " + pullLimit);
            log.info("===================================================");


            for (int i = 0, size = Integer.valueOf(clientSize); i < size; i++) {
                final String clientIdStr = clientId;
                try {
                    DdqH2Client h2Client = new DdqH2ClientBuilder().setHost(host).setPort(port).setHeartbeatInterval(100000)
                            .setHeartbeatTimeoutThreshold(300000).setClientMode(ClientModeType.Pull)
                            .setPullLimit(pullLimit)
                            .setPullInterval(pullPeriod)
                            .setClientId(clientIdStr)
                            .setAppId(appId)
                            .setAppSecret(appSecret)
                            .setAuthType(authType)
                            .setAuthHost(authHost)
                            .setAutoAck(true)
                            .setClientMode(ClientModeType.Pull)
                            .setNettyIoThreadNum(Integer.valueOf(threadNum))
                            .setGroupId(groupId)
                            .build();
                    h2Client.addListener(new DataReceiverDisplay());
                    h2Client.start();
                } catch (Throwable e) {
                    log.error("{} connection failed", clientIdStr, e);
                    System.exit(-1);
                }
            }
        } catch (Throwable e) {
            log.error("Http2ClientCmdTools error", e);
        }

    }

    /**
     * 具体的处理函数
     */
    public static class DataReceiverDisplay implements DdqClientMessageHandler {
        @Override
        public void messageRead(String topic, String messageId, JSONObject payload) {
            /* 以物模型属性上报为例子：
             * topic=/service/device-biz/attribute，messageId=消息的唯一ID
             * payload内容是->
             * {
                    "version": 1,
                    "time": "触发时间",
                    "type": "service",
                    "id": "设备唯一ID",
                    "corp_id": "企业ID",
                    "iot": {
                        "id": "IoT设备ID",
                        "mac": "IoT设备MAC",
                        "product": {
                            "id": "IoT产品ID"
                        },
                        "attribute": [{
                            "exception_info": {
                                "time": 1642*****0036,
                                "status": 2
                            },
                            "index": 1,
                            "field": "mode",
                            "value": "close"
                        }]
                    },
                    "project": {
                        "id": "项目ID"
                    }
                }
             */
            log.debug("dataPayload {}", payload);
            String     corpId     = payload.getString("corp_id");
            JSONObject iot        = payload.getJSONObject("iot");
            int        deviceId   = iot.getInteger("id");
            String     mac        = iot.getString("mac");
            JSONArray  attributes = iot.getJSONArray("attribute");
            for (int i = 0, size = attributes.size(); i < size; i++) {
                JSONObject item      = attributes.getJSONObject(i);
                String     fieldName = item.getString("field");
                Object     value     = item.get("value");
            }
        }
    }
}