package cn.xlink.test;

import cn.xlink.http2.client.Http2Client;
import cn.xlink.http2.client.callback.AbstractHttp2StreamDataReceiver;
import cn.xlink.http2.client.callback.Http2StreamListener;
import cn.xlink.http2.client.connection.Connection;
import cn.xlink.http2.client.connection.ConnectionListener;
import cn.xlink.http2.client.connection.ConnectionStatus;
import cn.xlink.http2.client.entity.Http2Response;
import cn.xlink.http2.client.entity.StreamData;
import com.alibaba.fastjson.JSONObject;
import io.netty.handler.codec.http2.DefaultHttp2Headers;
import io.netty.handler.codec.http2.Http2Headers;
import io.netty.handler.codec.http2.Http2Settings;
import io.netty.handler.codec.http2.Http2Stream;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.util.concurrent.ExecutionException;

/**
 * @Author shenweiran
 * @Date 2020-01-07 21:53
 */
public class TestClient {

    private static final Logger log = LoggerFactory.getLogger(TestClient.class);

    //目标服务器IP或域名
    static String            host          = "xxx";
    //static String host = "52.83.154.172";
    //目标服务服务器
    static int               port          = 80;
    //认证url
    static String            authUrl       = "/ddq/auth";
    //订阅主题url
    static String            subscribeUrl  = "/ddq/message/subscribe";
    //需要订阅的主题
    static String            topic         = "/device/datapoint/product/xxx/device/xxx";
    //外部授权ID
    static String            appId         = "xxxx";
    //外部授权及密钥得到的token
    static String            accessToken   = "xxx";
    //socket套接字对象
    static InetSocketAddress socketAddress = new InetSocketAddress(host, port);

    /**
     * VM Options：-Dlog4j.configurationFile=src/main/resources/log4j2.xml
     *
     * @param args
     * @throws ExecutionException
     * @throws InterruptedException
     */
    public static void main(String[] args) throws ExecutionException, InterruptedException {

        try {

            connect();

        } catch (Exception e) {
            log.error("", e);
            //连接失败，发起重连
            new ReconnectThread().start();
        }

    }

    private static void connect() throws Exception {
        // 创建http2客户端，构造参数1为心跳间隔，参数2为判断超时时间，单位皆为毫秒
        Http2Client http2Client = new Http2Client(1000, 15000);
        //建立远程连接
        Connection connection = http2Client.connect(socketAddress).get();
        //设置接收数据回调
        connection.setDefaultStreamListener(new Http2DataReceiver());
        connection.setConnectionListener(new Http2ConnectionListener());

        //客户端认证
        Http2Headers headers = new DefaultHttp2Headers();
        headers.path(authUrl).scheme("http").method("POST").authority("xlink.cn");
        //构建request data
        JSONObject body = new JSONObject();
        body.put("app_id", appId);
        body.put("access_token", accessToken);
        body.put("auth_type", "accesskey");
        byte[] data = body.toJSONString().getBytes();
        //发起认证请求
        connection.writeHeaders(headers, false, new Http2StreamListener() {
            @Override
            public void onDataRead(Connection connection, Http2Stream streamData, byte[] data, boolean endOfStream) {
                String     dataString = new String(data);
                JSONObject json       = JSONObject.parseObject(dataString);
                log.info("info1 {} ", json.toJSONString());
            }

            @Override
            public void onHeadersRead(Connection connection, Http2Stream stream, Http2Headers headers, boolean endOfStream) {
                log.info("info {} ", connection.getStatus());
            }

            @Override
            public void onStreamError(Connection connection, Http2Stream stream, IOException e) {
                log.info("info {} ", connection.getStatus());
            }
        }).get().writeData(data, true).get();


        //订阅消息
        Http2Headers headersSub = new DefaultHttp2Headers();
        JSONObject   subData    = new JSONObject();
        subData.put("topic", topic);
        headersSub.path(subscribeUrl).scheme("http").method("POST").authority("xlink.cn");
        //发起订阅
        connection.writeHeaders(headersSub, false, new Http2StreamListener() {
            @Override
            public void onDataRead(Connection connection, Http2Stream stream, byte[] data, boolean endOfStream) {
                String     dataString = new String(data);
                JSONObject json       = JSONObject.parseObject(dataString);
                log.info("info {} ", json.toJSONString());
            }

            @Override
            public void onHeadersRead(Connection connection, Http2Stream stream, Http2Headers headers, boolean endOfStream) {
                log.info("info {} ", connection.getStatus());
            }

            @Override
            public void onStreamError(Connection connection, Http2Stream stream, IOException e) {
                log.info("info {} ", connection.getStatus());
            }
        }).get().writeData(subData.toJSONString().getBytes(), true).get();
    }


    /**
     * 重连线程
     */
    private static class ReconnectThread extends Thread {


        @Override
        public void run() {
            for (; ; ) {
                try {
                    //每5秒进行重连
                    Thread.sleep(5000L);
                    connect();
                    //没有异常说明连接成功，跳出定时重连
                    break;
                } catch (Exception e) {
                    log.error("", e);
                }
            }
        }
    }


    /**
     * http/2 连接状态监听
     */
    public static class Http2ConnectionListener implements ConnectionListener {

        @Override
        public void onSettingReceive(Connection connection, Http2Settings settings) {

        }

        @Override
        public void onStatusChange(ConnectionStatus status, Connection connection) {
            if (status == ConnectionStatus.CLOSED) {
                log.error("http/2 connection disconnected. ");
                connection.close();
                new ReconnectThread().start();
            }
        }
    }


    /**
     * 接收数据处理
     */
    public static class Http2DataReceiver extends AbstractHttp2StreamDataReceiver {

        @Override
        public void onDataRead(Connection connection, Http2Stream stream, StreamData streamData) {
            Http2Response response = new Http2Response(streamData.getHeaders(), streamData.readAllData());
            //接收到的数据
            String content = new String(response.getContent());
            log.info("receiver data {}", content);
            JSONObject json = JSONObject.parseObject(content);
            //...
            String     topic   = json.getString("topic");
            String     msgId   = json.getString("msgId");
            JSONObject payload = json.getJSONObject("payload");

        }

        @Override
        public void onStreamError(Connection connection, Http2Stream stream, IOException e) {

        }
    }

}
